unit TcpSrv1;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, WSocket, WSocketS, SrvProcess, Db,
  DBTables;

const
  WM_APPSTARTUP      = WM_USER + 1;

type
  TTcpSrvForm = class(TForm)
    DisplayMemo: TMemo;
    WSocketServer1: TWSocketServer;
    Database: TDatabase;
    ssGoyaz: TSession;
    procedure FormShow(Sender: TObject);
    procedure WSocketServer1ClientConnect(Sender: TObject;
      Client: TWSocketClient; Error: Word);
    procedure WSocketServer1ClientDisconnect(Sender: TObject;
      Client: TWSocketClient; Error: Word);
    procedure WSocketServer1BgException(Sender: TObject; E: Exception;
      var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    FInitialized : Boolean;
    procedure Display(Msg : String);
    procedure WMAppStartup(var Msg: TMessage); message WM_APPSTARTUP;
    procedure ClientDataAvailable(Sender: TObject; Error: Word);
    procedure ClientBgException(Sender       : TObject;
                                E            : Exception;
                                var CanClose : Boolean);
  public
  end;

var
  TcpSrvForm: TTcpSrvForm;

implementation

{$R *.DFM}

procedure TTcpSrvForm.FormShow(Sender: TObject);
begin
  if not FInitialized then begin
    FInitialized := TRUE;
    PostMessage(Handle, WM_APPSTARTUP, 0, 0);
  end;
end;

procedure TTcpSrvForm.Display(Msg : String);
var
  I : Integer;
begin
  DisplayMemo.Lines.BeginUpdate;
  try
    if DisplayMemo.Lines.Count > 200 then begin
      for I := 1 to 50 do
        DisplayMemo.Lines.Delete(0);
    end;
    DisplayMemo.Lines.Add(Msg);
  finally
    DisplayMemo.Lines.EndUpdate;
  end;
end;


procedure TTcpSrvForm.WMAppStartup(var Msg: TMessage);
begin
  DisplayMemo.Clear;
  Display('Starting Database...');
  Database.Open;
  WSocketServer1.Proto       := 'tcp';         { Use TCP protocol  }
  WSocketServer1.Port        := 'telnet';      { Use telnet port   }
  WSocketServer1.Addr        := '0.0.0.0';     { Use any interface }
  WSocketServer1.ClientClass := TSrvClient; { Use our component }
  WSocketServer1.Listen;                       { Start litening    }
  Display('Waiting for clients...');
end;

procedure TTcpSrvForm.WSocketServer1ClientConnect(
  Sender : TObject;
  Client : TWSocketClient;
  Error  : Word);
begin
  with Client as TSrvClient do begin
    Display('Client connecting: ' + PeerAddr);
    LineMode        := False;
    LineEdit        := False;
    OnDataAvailable := ClientDataAvailable;
    OnBgException   := ClientBgException;
    Buffer.SessionName  := 'ssGoyaz';
    Buffer.DatabaseName := 'dbGoyaz';
  end;
end;

procedure TTcpSrvForm.WSocketServer1ClientDisconnect(
  Sender : TObject;
  Client : TWSocketClient;
  Error  : Word);
begin
  with Client as TSrvClient do begin
    Display('Client disconnecting: ' + PeerAddr + '   ' +
      'Duration: ' + FormatDateTime('hh:nn:ss',
      Now - Buffer.LoginTime));
  end;
end;

procedure TTcpSrvForm.ClientDataAvailable(
  Sender : TObject;
  Error  : Word);
begin
  (Sender as TSrvClient).Buffer.Parse;
end;


procedure TTcpSrvForm.WSocketServer1BgException(
  Sender       : TObject;
  E            : Exception;
  var CanClose : Boolean);
begin
  Display('Server exception occured: ' + E.ClassName + ': ' + E.Message);
  CanClose := FALSE;  { Hoping that server will still work ! }
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{ This event handler is called when a client socket experience a background }
{ exception. It is likely to occurs when client aborted connection and data }
{ has not been sent yet.                                                    }
procedure TTcpSrvForm.ClientBgException(
    Sender       : TObject;
    E            : Exception;
    var CanClose : Boolean);
begin
  Display('Client exception occured: ' + E.ClassName + ': ' + E.Message);
  CanClose := TRUE;   { Goodbye client ! }
end;

procedure TTcpSrvForm.FormCreate(Sender: TObject);
begin
  ServerLog := DisplayMemo.Lines;
end;

procedure TTcpSrvForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ServerLog := nil;
end;

end.

